// LEDSwitchShield Library
// Copyright (c) 2014 by Haute Solutions / Ed Van Every (www.haute-solutions.com)
//
// This Library is intended to support simple LED Display and Control
// using the Haute Solutions LEDSwitchShield
//
// This file is free software; you can redistribute it and/or modify
// it under the terms of either the GNU General Public License version 2
// or the GNU Lesser General Public License version 2.1, both as
// published by the Free Software Foundation.
//

#include <LEDSwitchShield.h>

int LEDSwitchShield_Matrix_Std[3][6] = {					// These are the "standard" Arduino pin assignments
        {  8,  9, 10, 11, 12, 13 },								// for use with the shield
        {  2,  3,  4,  5,  6,  7 },
        { A0, A1, A2, A3, A4, A5 }
};

int LEDSwitchShield_Matrix_Egg[3][6] = {					// These are the "alternate" pin assigments which
        {  8,  9, 49, 11, 12, 13 },								// Should be used if the shield is used with the
        {  2,  3, 46,  5, 47, 48 },								// EGGBoard and alternate I/O pins have been
        { A0, A1, A2, A3, A4, A5 }								// selected for I/O to the shield
};

int LEDSwitchShield_Numbers[13][3] = {
        { B111110, B100010, B111110 }, 						// 0
        { B000000, B000000, B111110 }, 						// 1
        { B101110, B101010, B111010 }, 						// 2
        { B101010, B101010, B111110 }, 						// 3
        { B111000, B001000, B111110 }, 						// 4
        { B111010, B101010, B101110 }, 						// 5
        { B111110, B101010, B101110 }, 						// 6
        { B100000, B100000, B111110 }, 						// 7
        { B111110, B101010, B111110 }, 						// 8
        { B111000, B101000, B111110 }, 						// 9
        { B000000, B000000, B000010 }, 						// Dot
        { B000000, B000000, B000000 }, 						// {None}
        { B111111, B111111, B111111 } 						// {All}
};

LEDSWITCHSHIELD::LEDSWITCHSHIELD(int mode){				// Call the constructor with mode=0 to assign standard I/O pins
  for(int i=0; i<3; i++) {												// and mode=1 to assign EGGBoard alternate pins
    for(int j=0; j<6; j++) {
    	switch (mode) {
    		case 1:
    			LEDSwitchShield_Matrix[i][j] = LEDSwitchShield_Matrix_Egg[i][j];
    			break;
    		default:
    			LEDSwitchShield_Matrix[i][j] = LEDSwitchShield_Matrix_Std[i][j];
    	}
      pinMode(LEDSwitchShield_Matrix[i][j], OUTPUT);
    }
  }
}

void LEDSWITCHSHIELD::DisplayIP(char *ip, int msdelay, int times) {

  for(int j=0; j<times; j++) {  
    DisplayAll();
    delay(msdelay);
    
    DisplayOff();  
    delay(msdelay);
    
    for(int i=0; i<strlen(ip); i++) {
      if(ip[i] == '.') DisplayDot();
      else DisplayNum(ip[i] - 48);
      delay(msdelay);
 
      DisplayOff();
      delay(msdelay);
    }
    
  }
  
  DisplayAll();
  delay(msdelay);
  DisplayOff();
}

void LEDSWITCHSHIELD::DisplayNum(int num) {

  DisplayColumn(0, LEDSwitchShield_Numbers[num][0]);
  DisplayColumn(1, LEDSwitchShield_Numbers[num][1]);
  DisplayColumn(2, LEDSwitchShield_Numbers[num][2]);  
}

void LEDSWITCHSHIELD::DisplayDot() {

  DisplayColumn(0, LEDSwitchShield_Numbers[10][0]);
  DisplayColumn(1, LEDSwitchShield_Numbers[10][1]);
  DisplayColumn(2, LEDSwitchShield_Numbers[10][2]);  
}

void LEDSWITCHSHIELD::DisplayOff() {

  DisplayColumn(0, LEDSwitchShield_Numbers[11][0]);
  DisplayColumn(1, LEDSwitchShield_Numbers[11][1]);
  DisplayColumn(2, LEDSwitchShield_Numbers[11][2]);  
}

void LEDSWITCHSHIELD::DisplayAll() {

  DisplayColumn(0, LEDSwitchShield_Numbers[12][0]);
  DisplayColumn(1, LEDSwitchShield_Numbers[12][1]);
  DisplayColumn(2, LEDSwitchShield_Numbers[12][2]);  
}

void LEDSWITCHSHIELD::DisplayColumn(int column, int number) {
int apin;
int mask = B000001;

  for(int i=0; i<6; i++) {
    apin = LEDSwitchShield_Matrix[column][i];
    if (number & mask) digitalWrite(apin, HIGH);
    else digitalWrite(apin, LOW);  
    mask = mask << 1;
  } 
}

void LEDSWITCHSHIELD::LEDControl(int x, int y, int func, int ms) {
int apin;

  apin = LEDSwitchShield_Matrix[x-1][y-1];
  
	if(func == 0) digitalWrite(apin, LOW);
	else if (ms > 0) {
		for(int i=0; i<func; i++) {
			digitalWrite(apin, HIGH);
			delay(ms);
			digitalWrite(apin, LOW);
			delay(ms);
  	} 
	} else digitalWrite(apin, HIGH);
}

void LEDSWITCHSHIELD::ToggleLED(int x, int y) {
int apin;

  apin = LEDSwitchShield_Matrix[x-1][y-1];
  
  if(digitalRead(apin) == LOW) digitalWrite(apin, HIGH);
  else digitalWrite(apin, LOW);
}

void LEDSWITCHSHIELD::ColumnUp(int x, int mode, int ms) {
int apin;

  for(int y=0; y<6; y++) {
    apin = LEDSwitchShield_Matrix[x-1][y];
    if (mode == 0) digitalWrite(apin, LOW);
    else digitalWrite(apin, HIGH);  
    delay(ms);
  } 
}

void LEDSWITCHSHIELD::ColumnDown(int x, int mode, int ms) {
int apin;

  for(int y=5; y>=0; y--) {
    apin = LEDSwitchShield_Matrix[x-1][y];
    if (mode == 0) digitalWrite(apin, LOW);
    else digitalWrite(apin, HIGH);  
    delay(ms);
  } 
}

int LEDSWITCHSHIELD::GetPin(int x, int y) {

  return LEDSwitchShield_Matrix[x-1][y-1];
}


